/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.util.Assert;

/**
 * Convenience Site that caches a username and password in the Core keyring file.
 * The keyring file is hard but not impossible to unscramble.
 * Hence, this class should only be used in those
 * cases where the username and password are not sensitive.
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under 
 * development and expected to change significantly before reaching stability. 
 * It is being made available at this early stage to solicit feedback from pioneering 
 * adopters on the understanding that any code that uses this API will almost 
 * certainly be broken (repeatedly) as the API evolves.
 * </p>
 */
public abstract class AuthenticatedSite extends Site {
	
	private String username;
	private String password;

	/**
	 * Return the location of this site in URL form.
	 * This method creates a URL from the location
	 * string obtained from {@link Site#getLocation()}.
	 * Subclasses must override if the location string
	 * is not a valid URL.
	 * @return the location of this site in URL form.
	 */
	public URL toUrl() {
		try {
			return new URL(getLocation());
		} catch (MalformedURLException e) {
			Assert.isTrue(false, "The location of this site is not a valid URL"); //$NON-NLS-1$
			// This return statement is never reached
			return null;
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.target.Site#discarded()
	 */
	public void discarded() throws CoreException {
		Platform.flushAuthorizationInfo(toUrl(), "", ""); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * Return the username used to connect ot this site.
	 * @return the username
	 */
	public String getUsername() {
		return username;
	}

	/**
	 * Return the password used to connect ot this site.
	 * @return the password
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * Set the username used to connect to this site
	 * @param name the username
	 * @throws CoreException
	 */
	public void setUsername(String name) throws CoreException {
		Map authInfo=Platform.getAuthorizationInfo(toUrl(), "", ""); //$NON-NLS-1$ //$NON-NLS-2$
		if (authInfo==null) authInfo=new HashMap(2);
		authInfo.put("name", name); //$NON-NLS-1$
		Platform.flushAuthorizationInfo(toUrl(), "", ""); //$NON-NLS-1$ //$NON-NLS-2$
		Platform.addAuthorizationInfo(toUrl(), "", "", authInfo); //$NON-NLS-1$ //$NON-NLS-2$
		this.username=name;
	}

	/**
	 * Set the password used to connect to this site
	 * @param password the password
	 * @throws CoreException
	 */
	public void setPassword(String password) throws CoreException {
		Map authInfo=Platform.getAuthorizationInfo(toUrl(), "", ""); //$NON-NLS-1$ //$NON-NLS-2$
		if (authInfo==null) authInfo=new HashMap(2);
		authInfo.put("password", password); //$NON-NLS-1${
		Platform.flushAuthorizationInfo(toUrl(), "", ""); //$NON-NLS-1$ //$NON-NLS-2$
		Platform.addAuthorizationInfo(toUrl(), "", "", authInfo); //$NON-NLS-1$ //$NON-NLS-2$
		this.password=password;
	}
	
}
