/*******************************************************************************
 * Copyright (c) 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.util.*;

import org.eclipse.core.runtime.*;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.target.*;
import org.eclipse.target.internal.ui.TargetAdapterFactory;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class TargetPlugin extends AbstractUIPlugin {
	
	public static final String ID = "org.eclipse.target"; //$NON-NLS-1$
	
	//The shared instance.
	private static TargetPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
	
	// image paths
	public static final String ICON_PATH = "icons/full/"; //$NON-NLS-1$
	
	private Map imageDescriptors = new HashMap(20);
	
	/**
	 * The constructor.
	 */
	public TargetPlugin() {
		super();
		plugin = this;
		try {
			resourceBundle = ResourceBundle.getBundle("org.eclipse.target.internal.messages"); //$NON-NLS-1$
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		
		TargetAdapterFactory factory = new TargetAdapterFactory();
		Platform.getAdapterManager().registerAdapters(factory, Site.class);
		Platform.getAdapterManager().registerAdapters(factory, ISiteManager.class);
		Platform.getAdapterManager().registerAdapters(factory, ITargetResource.class);
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
	}

	/**
	 * Returns the shared instance.
	 */
	public static TargetPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

	public static void log(IStatus status) {
		getDefault().getLog().log(status);
		
	}

	public static void log(CoreException e) {
		log (e.getMessage(), e);
	}

	public static void log(String string, CoreException e) {
		log(new Status(e.getStatus().getCode(), ID, 0, string, e));
	}
	
	public static void log(IOException e) {
		getDefault().getLog().log(new Status(IStatus.ERROR, ID, 0, e.getMessage(), e));
	}
	
	public static IProgressMonitor subMonitorFor(IProgressMonitor monitor, int i) {
		return new SubProgressMonitor(monitor, i);
	}

	/**
	 * Create an imag descriptor for the image that is located at the given location
	 * relative to the icon location of the Target plugin.
	 * @param location the location of the icon relative to the Target plugin
	 * @return an image descriptor
	 */
	public ImageDescriptor getImageDescriptor(String id) {
		if(! imageDescriptors.containsKey(id)) {
			URL imageURL = TargetPlugin.getDefault().getBundle().getEntry(ICON_PATH + id);
			createImageDescriptor(id, imageURL);
		}
		return (ImageDescriptor)imageDescriptors.get(id);
	}
	
	/**
	 * Return the cached image descriptor for the image located at the given 
	 * URL. The first time the method is called for the given id, the descriptor
	 * will be cached. Subsequent calls will return the cached descriptor.
	 * @param id the id that uniquely identifies the image
	 * @param imageURL the URL of the image
	 * @return  an image descriptor
	 */
	public ImageDescriptor getImageDescriptor(String id, URL imageURL) {
		if(! imageDescriptors.containsKey(id)) {
			createImageDescriptor(id, imageURL);
		}
		return (ImageDescriptor)imageDescriptors.get(id);
	}
	
	private void createImageDescriptor(String id, URL url) {
		ImageDescriptor desc = ImageDescriptor.createFromURL(url);
		imageDescriptors.put(id, desc);
	}

	public static void checkCanceled(IProgressMonitor monitor) {
		if (monitor != null && monitor.isCanceled()) {
			throw new OperationCanceledException();
		}
		
	}

	public static CoreException asCoreException(InvocationTargetException e) {
		if (e.getTargetException() instanceof CoreException) {
			return (CoreException)e.getTargetException();
		}
		return new CoreException(new Status(IStatus.ERROR, ID, 0, "An internal error occurred: " + e.getMessage(), e));
	}
}
