/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.ui;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.*;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.wizard.*;
import org.eclipse.osgi.util.NLS;
import org.eclipse.target.ITargetResource;
import org.eclipse.target.Site;
import org.eclipse.target.internal.*;
import org.eclipse.target.internal.ITargetImages;
import org.eclipse.target.internal.TargetPlugin;
import org.eclipse.target.internal.registry.SiteCreationWizardsRegistry;
import org.eclipse.target.internal.registry.SiteWizardDescriptor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.model.AdaptableList;

public class CreateSiteWizard extends Wizard {
	protected IWorkbench workbench;
	protected Wizard wizard;
	
	protected CreateSiteWizardMainPage mainPage;
	
	public CreateSiteWizard() {
		setWindowTitle(getWizardWindowTitle()); //$NON-NLS-1$
	}
	
	/**
	 * @see ConfigureProjectWizard#getWizardDescription()
	 */
	protected String getWizardDescription() {
		return TargetMessages.TargetSiteCreationWizard_description; //$NON-NLS-1$
	}

	/**
	 * @see ConfigureProjectWizard#getWizardLabel()
	 */
	protected String getWizardLabel() {
		return TargetMessages.TargetSiteCreationWizard_label; //$NON-NLS-1$
	}

	/**
	 * @see ConfigureProjectWizard#getWizardWindowTitle()
	 */
	protected String getWizardWindowTitle() {
		return TargetMessages.TargetSiteCreationWizard_windowTitleNoProject; //$NON-NLS-1$
	}
	
	/*
	 * @see Wizard#addPages
	 */
	public void addPages() {
		AdaptableList wizards = getAvailableWizards();
		try {
			if(wizards.size() == 1) {
				SiteWizardDescriptor element = (SiteWizardDescriptor)wizards.getChildren()[0];
				this.wizard = element.createWizard();
				wizard.addPages();
				if (wizard.getPageCount() > 0) {
					wizard.setContainer(getContainer());
					IWizardPage[] pages = wizard.getPages();
					for (int i = 0; i < pages.length; i++) {
						addPage(pages[i]);
					}
				}
			} else {
				setForcePreviousAndNextButtons(true);
				mainPage = new CreateSiteWizardMainPage("target-selection-page", getWizardLabel(), TargetPlugin.getDefault().getImageDescriptor(ITargetImages.IMG_WIZBAN_CREATE_SITE), wizards, new AdaptableList(), TargetMessages.ConfigureProjectWizardMainPage_selectTarget); //$NON-NLS-1$ //$NON-NLS-2$
				mainPage.setDescription(getWizardDescription());
				addPage(mainPage);
			}
		} catch (CoreException e) {
			TargetPlugin.log(e);
			return;
		}
	}
	
	public IWizardPage getNextPage(IWizardPage page) {
		if(wizard != null) {
			return wizard.getNextPage(page);
		}
		return super.getNextPage(page);
	}
	
	public boolean canFinish() {
		// If we are on the first page, never allow finish unless the selected wizard has no pages.
		IWizardPage currentPage = getContainer().getCurrentPage();
		if (currentPage == mainPage) {
			if (mainPage.getSelectedWizard() != null && mainPage.getNextPage() == null) {
				return true;
			}
			return false;
		}
		if(wizard != null) {
			return wizard.canFinish();
		}
		return super.canFinish();
	}
	
	/*
	 * @see Wizard#performFinish
	 */
	public boolean performFinish() {
		// allow target wizard to finish
		if (wizard != null) {
			return wizard.performFinish();
		}
		// If we are on the first page and the selected wizard has no pages then
		// allow it to finish.
		if (getContainer().getCurrentPage() == mainPage) {
			Wizard noPageWizard = mainPage.getSelectedWizard();
			if (noPageWizard != null) {
				if (noPageWizard.canFinish()) 
				{
					return noPageWizard.performFinish();
				}
			}
		}
		return true;
	}
	
	public static boolean validateSite(final Site site, final IWizardContainer container) {
		final boolean[] valid = new boolean[] {true};
		final String[] message = new String[] {TargetMessages.ConfigureTargetWizardURL_doesn__t_exist_on_the_server_8}; //$NON-NLS-1$
		final int[] code = new int[] {-1};
		try {
			container.run(true, false, new IRunnableWithProgress() {
				public void run(IProgressMonitor monitor)
					throws InvocationTargetException, InterruptedException {
						try {
							monitor.beginTask(TargetMessages.ConfigureTargetWizardValidating_connection_to_Site____9, IProgressMonitor.UNKNOWN); //$NON-NLS-1$
							ITargetResource remote = site.getRootResource();
							valid[0] = remote.getSite().canBeReached(monitor);
						} catch(CoreException e) {
							message[0] = e.getStatus().getMessage();
							code[0] = e.getStatus().getCode();
							valid[0] = false;
						} finally {
							monitor.done();
						}
				}
			});
		} catch (InvocationTargetException e) {
			valid[0] = false;
			message[0] = e.getTargetException().getMessage();
		} catch (InterruptedException e) {
			return false;
		}
		if(! valid[0]) {
			if(! MessageDialog.openQuestion(container.getShell(),
				TargetMessages.ConfigureTargetWizardConnection_Error_10, //$NON-NLS-1$
				NLS.bind(TargetMessages.ConfigureTargetWizard_errorOccurred, (new Object[] {site.getLocation(), new Integer(code[0]), message[0]})))) { //$NON-NLS-1$
					return false;
			}
		}
		return true;		
	}
	
	/**
	 * Returns the configuration wizards that are available for invocation.
	 * 
	 * @return the available wizards
	 */
	protected AdaptableList getAvailableWizards() {
		SiteCreationWizardsRegistry registry = new SiteCreationWizardsRegistry();
		registry.readRegistry(Platform.getExtensionRegistry(), TargetPlugin.ID, SiteCreationWizardsRegistry.PT_SITE_CREATION_WIZARDS);
		return registry.asAdaptableList();
	}
}
