/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.ui.actions;

import java.lang.reflect.InvocationTargetException;
import java.net.URL;

import org.eclipse.core.runtime.*;
import org.eclipse.core.runtime.jobs.*;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.target.internal.TargetPlugin;
import org.eclipse.target.internal.Utils;
import org.eclipse.ui.IWorkbenchSite;
import org.eclipse.ui.progress.IProgressConstants;

/**
 * This runnable context executes it's operation in the context of a background job.
 */
public final class JobRunnableContext implements ITargetRunnableContext {

	private IJobChangeListener listener;
	private IWorkbenchSite site;
	private String jobName;
	private ISchedulingRule schedulingRule;
	private boolean isUser;
	private URL icon;
	private boolean keep;
	private IAction gotoAction;

	public JobRunnableContext(String jobName) {
		this(jobName, null, null, false, null, null);
	}
	
	public JobRunnableContext(String jobName, URL icon, IAction action, boolean keep, IJobChangeListener listener, IWorkbenchSite site) {
		this.jobName = jobName;
		this.listener = listener;
		this.site = site;
		// By default team actions are user initiated. 
		this.isUser = true;
		this.gotoAction = action;
		this.icon = icon;
		this.keep = keep;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.internal.ccvs.ui.operations.ITeamRunnableContext#run(java.lang.String, boolean, org.eclipse.jface.operation.IRunnableWithProgress)
	 */
	public void run(IRunnableWithProgress runnable) {
		Job job= getBasicJob(runnable);
		if (schedulingRule != null) {
			job.setRule(schedulingRule);
		}
		if (listener != null) {
			job.addJobChangeListener(listener);
		}
		job.setUser(isUser());
		configureJob(job);
		Utils.schedule(job, site);
	}
	
	private void configureJob(Job job) {
		if(keep)
			job.setProperty(IProgressConstants.KEEP_PROPERTY, Boolean.TRUE); 
		if(gotoAction != null)
			job.setProperty(IProgressConstants.ACTION_PROPERTY, gotoAction);
		if(icon != null)
			job.setProperty(IProgressConstants.ICON_PROPERTY, icon);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.internal.ccvs.ui.operations.ITeamRunnableContext#getShell()
	 */
	public Shell getShell() {
		final Shell[] newShell = new Shell[] { null };
		Display.getDefault().syncExec(
			new Runnable() {
				public void run() {
					newShell[0] = Utils.getShell(site);
				}
			});
		return newShell[0];
	}
	
	/**
	 * Return the scheduling rule that will be obtained before the context
	 * executes a runnable or <code>null</code> if no scheduling rule is to be onbtained.
	 * @return the schedulingRule to be obtained or <code>null</code>.
	 */
	public ISchedulingRule getSchedulingRule() {
		return schedulingRule;
	}
	
	/**
	 * Return whether this job context is user initiated.
	 * @param boolean <code>true</code> if the job is a result of a user initiated actions
	 * and <code>false</code> otherwise.
	 */
	public boolean isUser() {
		return isUser;
	}
	
	/**
	 * Set wheter the job is user initiated. By default the job created by this runnable context
	 * is a user job.
	 * @param isUser <code>true</code> if the job is a result of a user initiated actions
	 * and <code>false</code> otherwise.
	 */
	public void setUser(boolean isUser) {
		this.isUser = isUser;
	}
	
	/**
	 * Set the scheduling rule that will be obtained before the context
	 * executes a runnable or <code>null</code> if no scheduling rule is to be onbtained.
	 * @param schedulingRule The schedulingRule to be obtained or <code>null</code>.
	 */
	public void setSchedulingRule(ISchedulingRule schedulingRule) {
		this.schedulingRule = schedulingRule;
	}
	
	/* private */ IStatus run(IRunnableWithProgress runnable, IProgressMonitor monitor) {
		try {
			runnable.run(monitor);
		} catch (InvocationTargetException e) {
			return TargetPlugin.asCoreException(e).getStatus();
		} catch (InterruptedException e) {
			return Status.OK_STATUS;
		}
		return Status.OK_STATUS;
	}
	
	private Job getBasicJob(final IRunnableWithProgress runnable) {
		return new Job(jobName) {
			public IStatus run(IProgressMonitor monitor) {
				return JobRunnableContext.this.run(runnable, monitor);
			}
		};
	}
}
