/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.ui.dnd;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.target.ITargetResource;
import org.eclipse.target.internal.Utils;
import org.eclipse.target.internal.ui.actions.CopyOperation;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.PluginDropAdapter;

/**
 * This class provides drop support to a Viewer whose elements are <code>ITargetResource</code>.
 */
public class TargetResourceViewerDropAdapter extends PluginDropAdapter {

	private final IWorkbenchPart part;
	
	public TargetResourceViewerDropAdapter(IWorkbenchPart part, StructuredViewer viewer) {
		super(viewer);
		this.part = part;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ViewerDropAdapter#performDrop(java.lang.Object)
	 */
	public boolean performDrop(Object data) {
		ITargetResource target = getTargetTargetResource();
		if (target != null) {
			if (data instanceof ITargetResource[]) {
				ITargetResource[] toDrop = (ITargetResource[]) data;
				//cannot drop a resource onto itself or a child
				for (int i = 0; i < toDrop.length; i++)
					if (toDrop[i].equals(target) || toDrop[i].getFullPath().startsWith(target.getFullPath()))
						return false;
				try {
					// Run the copy in the foreground
					new CopyOperation(part, target, toDrop) {
						protected boolean canRunAsJob() {
							return false;
						}
					}.run();
				} catch (InvocationTargetException e) {
					Utils.handle(part.getSite().getShell(), e);
					return false;
				} catch (InterruptedException e) {
					return false;
				}
				refreshViewer(target, toDrop);
				return true;
			} else if (data instanceof String[]){
				// The strings are absolute file paths from a FileTransfer
				// TODO: implement file drop 
			}
		}
		return false;
	}

	/**
	 * Refresh the viewer for the given target and dragged resources.
	 * @param target the target folder
	 * @param toDrop the resources tht were copied
	 */
	protected void refreshViewer(ITargetResource target, ITargetResource[] toDrop) {
		getViewer().refresh();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ViewerDropAdapter#validateDrop(java.lang.Object, int, org.eclipse.swt.dnd.TransferData)
	 */
	public boolean validateDrop(Object target, int operation, TransferData transferType) {
		if (TargetResourceTransfer.getInstance().isSupportedType(transferType)
				&& getTargetDirectory(target) != null) {
			return true;
		}
		return super.validateDrop(target, operation, transferType);
	}

	protected ITargetResource getTargetTargetResource() {
		Object target = getCurrentTarget();
		if (target == null)
			target = getViewer().getInput();
		return getTargetDirectory(target);
	}

	protected ITargetResource getTargetDirectory(Object target) {
		ITargetResource resource = Utils.asResource(target);
		if (resource != null) {
			switch (getCurrentLocation()) {
				case LOCATION_AFTER:
				case LOCATION_BEFORE:
					resource = resource.getParent();
					break;
				default:
					break;
			}
			if (resource != null && resource.isDirectory()) 
				return resource;
		}
		return null;
	}
}
