/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.internal.target.ui;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.target.Site;
import org.eclipse.target.ui.SiteCreationWizardPage;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.internal.target.TeamTargetMessages;
import org.eclipse.team.internal.ui.ITeamUIImages;
import org.eclipse.team.internal.ui.TeamUIPlugin;

public abstract class ExportWizard extends DeploymentWizard {

	private SiteCreationWizardPage siteCreationPage;
	private SiteSelectionPage siteSelectionPage;
	private ExportSourceSelectionPage containerSelectionPage;
	private MappingSelectionPage mappingPage;
	private ExportResourceSelectionPage resourceSelectionPage;
	
	public void addPages() {
		containerSelectionPage = new ExportSourceSelectionPage(
				"container-selection",  //$NON-NLS-1$
				TeamTargetMessages.ExportWizard_1,  //$NON-NLS-1$
				TeamTargetMessages.ExportWizard_2,  //$NON-NLS-1$
				TeamUIPlugin.getImageDescriptor(ITeamUIImages.IMG_WIZBAN_SHARE),
				getDeploymentProviderId());
		//siteSelectionPage.setDialogSettings(getDialogSettings());
		containerSelectionPage.setInitialSelection(selection);
		addPage(containerSelectionPage);
		
		siteSelectionPage = new SiteSelectionPage(
				"site-selection",  //$NON-NLS-1$
				TeamTargetMessages.ExportWizard_4,  //$NON-NLS-1$
				TeamTargetMessages.ExportWizard_5,  //$NON-NLS-1$
				TeamUIPlugin.getImageDescriptor(ITeamUIImages.IMG_WIZBAN_SHARE), 
				getSiteType());
		//siteSelectionPage.setDialogSettings(getDialogSettings());
		addPage(siteSelectionPage);
		
		siteCreationPage = createSiteCreationPage();
		addPage(siteCreationPage);
		
		mappingPage = new MappingSelectionPage(
				"site-mapping",  //$NON-NLS-1$
				TeamTargetMessages.ExportWizard_7, //$NON-NLS-1$
				TeamTargetMessages.ExportWizard_8, //$NON-NLS-1$
				TeamUIPlugin.getImageDescriptor(ITeamUIImages.IMG_WIZBAN_SHARE));
		//mappingPage.setDialogSettings(getDialogSettings());
		addPage(mappingPage);
		
		resourceSelectionPage = new ExportResourceSelectionPage(
				"resource-selection", //$NON-NLS-1$
				TeamTargetMessages.ExportWizard_10, //$NON-NLS-1$
				TeamTargetMessages.ExportWizard_11, //$NON-NLS-1$
				TeamUIPlugin.getImageDescriptor(ITeamUIImages.IMG_WIZBAN_SHARE),
				this);
		addPage(resourceSelectionPage);
	}
	
	protected abstract SiteCreationWizardPage createSiteCreationPage();

	public IWizardPage getNextPage(IWizardPage page) {
		if (page == containerSelectionPage) {
			if (containerSelectionPage.isNewDeployment()) {
				return siteSelectionPage;
			} else {
				return resourceSelectionPage;
			}
		}
		if (page == siteSelectionPage) {
			if(siteSelectionPage.getSite() != null) {
				mappingPage.setSite(siteSelectionPage.getSite());
				mappingPage.setContainer(containerSelectionPage.getSelectedContainer());
				mappingPage.setPreviousPage(page);
				return mappingPage;
			} else {
				return siteCreationPage;
			}
		}
		if (page == siteCreationPage) {
			mappingPage.setSite(siteCreationPage.getSite());
			mappingPage.setContainer(containerSelectionPage.getSelectedContainer());
			mappingPage.setPreviousPage(page);
			return mappingPage;
		}
		if (page == mappingPage) {
			return resourceSelectionPage;
		}
		return super.getNextPage(page);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 */
	public boolean performFinish() {
		// Make sure that the deployment provider has been created if needed
		if (createdDeploymentProvider == null) {
			// This may or may not create a deployment provider.
			getSelectedDeploymentProvider();
		}
 		try {
			final IResource[] selected = resourceSelectionPage.getSelectedResources();
			getContainer().run(true, true, new IRunnableWithProgress() {
				public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
					try {
						IStatus status = getSubscriber().getTeamOperations().put(selected, IResource.DEPTH_ZERO, true, monitor);
						if (!status.isOK()) {
							throw new TeamException(status);
						}
					} catch (TeamException e) {
						throw new InvocationTargetException(e);
					}
				}
			});
		} catch (InvocationTargetException e) {
			handle(e);
		} catch (InterruptedException e) {
			return false;
		}
		return true;
	}

	protected Site getSite() {
		Site site = siteSelectionPage.getSite();
		if (site == null) {
			site = siteCreationPage.getSite();
		}
		return site;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.IWizard#canFinish()
	 */
	public boolean canFinish() {
		if (containerSelectionPage.isPageComplete() &&
				siteSelectionPage.isPageComplete() &&
				mappingPage.isPageComplete() &&
				getSite() != null &&
				resourceSelectionPage.isPageComplete()) {
			return true;
		}
		return super.canFinish();
	}

	protected boolean isNewDeployment() {
		return containerSelectionPage.isNewDeployment();
	}
	
	protected IContainer getSelectedContainer() {
		return containerSelectionPage.getSelectedContainer();
	}
	
	protected IPath getMapping() {
		return mappingPage.getMapping();
	}
}
